;
; Creation par AC le 17/12/2003 a Meudon
;
; Historique des modifications:
;
; le 14/01/2004: Michel Piat :
; - Modif de l'equation Pfuite
; - chgt de nom de plusieurs variables intermediares
;
; le 15/01/2004: Alain Coulais :
; - remise a plat des keywords d'I/O
;
; le 02/02/2004: AC (suite a Planck Consortium Meeting avec MP):
; - "d" devient "dcapa"
; - on ajoute "bolo" comme keyword
; - desormais, "background" s'ajoute a "signal"
;
; le 08/04/2004: AC (Orsay)
; - N.B.: Ce programme est base sur "Equa_Temp_Bolo"
; - le programme est renomme de EQUA_TEMP_BOLO en BOLO_DIRECT_MODEL
; - on passe de procedure en function
; - on supprime "background"
; - desormais "signal" est la somme de  1/ "signal optique",
;   2/ "background", 3/ "noise". Tout ceci doit etre pret en amont. 
; - remplacement de /bavard par /verbose
;
; Dependances (Ce programme necessite):
;      AC_SETMESSAGES 
;
function BOLO_DIRECT_MODEL, signal, delta_t=delta_t, $
                            current=current, $      ;; polarization current (array)
                            bolo=bolo, $            ;; Structure "bolo"
                            r0=r0, Tg=Tg, m=m, $    ;; parametres de P Joule
                            g_s0=g_s0, beta=beta, $ ;; parametres de P fuite
                            c0=c0, dcapa=dcapa, $   ;; parametres de P interne Bolo
                            display=display, verbose=verbose, $
                            debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('BOLO_DIRECT_MODEL')
;
if (N_PARAMS() lt 1) then begin
	print, mess.fatal+'missing arguments'
	help=1
endif
;
if KEYWORD_SET(help) then begin
    print, 'function BOLO_DIRECT_MODEL, signal, delta_t=delta_t, $'
    print, '                 current=current, $      ;; polarization current (array)'
    print, '                 bolo=bolo, $            ;; Structure "bolo"'
    print, '                 r0=r0, Tg=Tg, m=m, $    ;; parametres de P Joule'
    print, '                 g_s0=g_s0, beta=beta, $ ;; parametres de P fuite'
    print, '                 c0=c0, dcapa=dcapa, $   ;; parametres de P interne Bolo'
    print, '                 display=display, verbose=verbose, $'    
    print, '                 debug=debug, test=test, help=help'
    print, ' '
    print, 'Default values are provided for each parameter ! (/verbose to print them)'
    print, 'P Joule: params (r0, Tg, m)'
    print, 'P bolo : params (c0, dcapa)'
    print, 'P fuite: params (g_s0, beta)'
    return, -1
endif
;
if (N_ELEMENTS(delta_t) NE 1) then delta_t=1e-4
if (delta_t LE 0.) then begin
    print, mess.fatal+'"delta_t" must be positive'
    return, -1
endif
; -------------------------------------------------
; description de la Resistance (P Joule)
;
; valeur indiquee par M. Piat
; (Attention, notre definition est differente de celle dans Chanin/Torre 84)
if (N_ELEMENTS(Tg) NE 1) then Tg=16.54 ;en K
; Chanin Torre 1984 (p412b): (m=0.25, 0.5 ou 1)
if (N_ELEMENTS(m) NE 1) then m=.5
; valeur donnee par Sudiwala
if (N_ELEMENTS(r0) NE 1) then r0=120.48 ;en ohm
;
; -------------------------------------------------
; description de la P fuite
;
; en W/K pour T=Tbain
if (N_ELEMENTS(g_s0) NE 1) then g_s0=60.*1e-12
;
; Chanin Torre 1984 : beta ~ 3 (p 413b)
; pour le cas planck, on a 1.2 a 1.5
if (N_ELEMENTS(beta) NE 1) then beta=1.5
;
; -------------------------------------------------
; description de la capacite calorifique bolo
;
; "c0" is the capacity for temp==temp0 (0.1K)
if (N_ELEMENTS(c0) NE 1) then c0=0.8e-12 ; en J/K
; Chanin Torre 1984 (p418b)
; pour le cas planck, on a dcapa==2.
if (N_ELEMENTS(dcapa) NE 1) then dcapa=2.1
;
; si un "bolo" est passe, on a chaque champ rempli
; et on ecrase tout ce qui peut etre mis par param ...
;
; AC, modif des noms des variable le 03/03/04, d'apres XML IMO
;
if (N_ELEMENTS(bolo) GT 0) then begin
    r0=bolo.Rinf
    Tg=bolo.Ttrans
    m=bolo.Alpha
    g_s0=bolo.Gbol
    beta=bolo.Beta
    c0=bolo.c0
    dcapa=bolo.dcapa
    ;; ATTENTION: ceci veut dire que "current" est a priori tjrs
    ;; active par ici puisque le champ "bolo.i_polar" existe tjrs !
    ;; le 04/03/04: Je rajoute un petit test de warning !
    if (N_ELEMENTS(current) GT 0) then begin
        print, mess.warning+"Indeed, current is set by <<bolo.i_polar>>"
    endif
    current=bolo.i_polar
endif
;
; on peut arriver ici SANS current (si valeurs par default)
;
if (N_ELEMENTS(current) EQ 0) then begin
    print, mess.fatal+"missing mandatory parameter <<current>>"
    return, -1
endif
;
double_precision=1
if KEYWORD_SET(double_precision) then begin
    r0=double(r0)
    Tg=Double(Tg)
    g_s0=Double(g_s0)
    c0=Double(c0)
    current=Double(current)
endif
;
if KEYWORD_SET(verbose) then begin
    print, mess.info+'P resistance (Tg, m, r0):', Tg, m, r0
    print, mess.info+'P interne Bolo (c0, dcapa):', c0, dcapa
    print, mess.info+'P fuite (g_s0, beta):', g_s0, beta
    print, mess.info+'Current is:', current(0)
endif
;
nbp=N_ELEMENTS(signal)
;
;if (N_ELEMENTS(nbp) NE 1) then nbp=-1
;
; AC 040408 desormais inutile
; si "signal" est fournie, on veut le resortir non ajouter a "background"
; on recopie "signal" dans une var. temporaire pour la resortir "intacte"
; if (N_ELEMENTS(nbpSignal) GT 0) then signal_ref=signal
;
; dans le cas ou on a rien en entree --> on fixe "nbp"
; sinon, on verifie si "nbpSignal == nbp"
;
; if ((nbp GT 0) AND (nbpSignal GT 0)) then begin
;     print, mess.warning+'both "nbp" and "signal" are provided ("signal" used)'
;     nbp=nbpSignal
; endif else begin
;     if ((nbp LE 0) AND (nbpSignal LE 0)) then begin
;         nbp=10000
;     endif else begin
;         nbp=MAX([nbp,nbpSignal])
;     endelse
; endelse
;
; ---------------------------------------------------------------------
; Le courant
; si on ne fournit qu'une seule valeur, on le transforme en tableau
; il n'y a pas de probleme de signe puisque qu'on se sert de son carre
;
nbp_current=N_ELEMENTS(current)
if (nbp_current LE 0) then begin
    current=1e-9                ; en nA
    print, mess.info+"Default value of <<current>> is set to:", current
    tab_i=REPLICATE(current,nbp)
endif
if (nbp_current EQ 1) then begin
    txt="value of <<current>>"
    if OUT_OF_RANGE(current, min=1e-14, max=1e-8, txt) then begin
        print, mess.warning+"value of <<current>> is out of expected range", current
    endif
    tab_i=REPLICATE(current,nbp)
endif
if (nbp_current GT 1) then begin
    if (nbp_current NE nbp) then begin
        print, mess.fatal+" nbp_current <> nbp ! ", nbp_current, nbp
        return, -1
    endif
endif
;
; en sortie de ce "case" on a necessairement un "tableau" "tab_i"
;
r0i2=r0*tab_i*tab_i
;
; AC 040408 desormais inutile
; ---------------------------------------------------------------------
; la puissance "optique"
;
; un ordre de grandeur pour le background est de 1e-12 ; en W sur le detecteur
;
; if (N_ELEMENTS(background) EQ 1) then begin
;     if (background LT 0.0) then begin
;         print, mess.fatal+'background MUST be POSITIVE !'
;         return, -1
;     endif
;     if (background GT 1e-6) then begin
;         print, mess.warning+'background MAY be too HIGH (check delta_t)'
;     endif
;     signal=signal+REPLICATE(background,nbp)
; endif
;
; ---------------------------------------------------------------------
; si le mot-clef Background est actif, alors on a un signal genere ci-dessus
; et donc on ne passe pas ici.
;
; if (N_ELEMENTS(signal) EQ 0) AND (N_ELEMENTS(background) EQ 0) then begin
;     background=1e-12            ; en W sur le detecteur
;     signal=REPLICATE(background,nbp)
;     if (nbp GT 1001) then signal(1000)=signal(1000)+100e-12
;     if (nbp GT 2001) then signal(2000)=signal(2000)+1000e-12
;     if (nbp GT 3501) then signal(3400:3440)=signal(3400:3440)+100e-12
;endif
;
; ---------------------------------------------------------------------
; creation des tableaux de reference
;
Temp0=DOUBLE(0.1)
TempRef=REPLICATE(Temp0, nbp)
TempCalcul=DBLARR(nbp)
TempCalcul(0)=TempRef(0)
;
; AC, test Double Precision le 18/02/2004
; t=Delta_t*findgen(nbp)
t=Delta_t*Dindgen(nbp)
;
; ---------------------------------------------------------------------
;
; le calcul lui-meme, simple x(n+1)=x(n)+delta_t*Function(x(n), params)
;
;debug=1
if KEYWORD_SET(debug) then begin
    Pj_tab=Dblarr(nbp)
    C_tab=Dblarr(nbp)
    Pf_tab=Dblarr(nbp)
endif
;
for i=1L, (nbp-1L) do begin
    Temp=TempCalcul(i-1)
    Pj=r0i2(i)*EXP((Tg/Temp)^m)
    C=c0*(Temp/Temp0)^dcapa
    Pf=g_s0*Temp0/(1.+beta)*((Temp/Temp0)^(beta+1.)-1.)
    ;;
    ;; new way in order to increase numerical stability
    TempCalcul(i)=Temp+(Delta_t/c)*((Pj-Pf)+signal(i))
    ;;
    ;; old way (no clear difference evidenced)
    ;;    TempCalcul(i)=Temp+Delta_t*(Pj+signal(i)-Pf)/c
    ;;
    ;;	if KEYWORD_SET(verbose) then print, 'Pj, C, Pf, Ti', $
    ;;	 	 Pj, C, Pf, TempCalcul(i)
    if KEYWORD_SET(debug) then begin
        Pj_tab(i)=Pj
        C_tab(i)=C
        Pf_tab(i)=Pf
    endif
endfor
;
if KEYWORD_SET(display) then begin
    !P.multi=[0,0,2]
    window, xsize=900, ysize=900
    plot, t, signal, title='Incoming flux'
    plot, t, TempCalcul, title='Temperature variation'
    !p.multi=0
endif
;
; AC 040408 desormais inutile
; on recopie l'original ...
; Rappel: ci-dessous, signal=signal_ref+background ...
;if (N_ELEMENTS(nbpSignal) GT 0) then signal=signal_ref
;
if KEYWORD_SET(debug) then begin
    STOP
    ;;HELP
endif
;
if KEYWORD_SET(test) then STOP
;
return, TempCalcul
;
end
;
