;
; Creation par AC le 17/01/2003 a Meudon
;
; Historique des modifications:
;
; le 02/02/04: reprise complete par AC suite aux essais de MPiat
;              durant le Planck Consortium Meeting
; le 19/02/04: reprise complete par AC pour integration avec les
;              calculs de Jupiter
;
; Dependances (Ce programme necessite):
;      AC_SETMESSAGES 
;      LWIN
;
pro TEST_BEAM, bolo=bolo, background=background, nbp=nbp, $
               i_bolo=i_bolo, noise=noise, $
               postscript=postscript, xrange=xrange, $
               bavard=bavard, test=test, debug=debug, help=help
;
mess=AC_SETMESSAGES("TEST_BEAM")
;
if KEYWORD_SET(help) then begin
    print, 'pro TEST_BEAM, bolo=bolo, background=background, nbp=nbp, $'
    print, '               i_bolo=i_bolo, noise=noise, $'
    print, '               postscript=postscript, xrange=xrange, $'
    print, '               bavard=bavard, test=test, debug=debug, help=help'
    return
endif

; il faut TROIS info pour faire tourner ce programme:
; -E1: les infos d'un bolo
; -E2: les niveaux de fond (background)
; -E3: les valeurs de flux de Jupiter
;
; --------------------------------------
; -E1: les infos d'un bolo
; dans le cas ou AUCUN bolo n'est passe en arguement,
; on propose les bolo deja connus ...
;
if NOT(KEYWORD_SET(bolo)) then begin
    print, mess.warning+"because you do not provide a bolo structure "
    print, mess.warning+"(cf BOLO_INIT) we try to help you to select one."
    ;;
    ;; a-t-on deja initilise les bolos (liste et valeurs de chacun ?)
    ;;
    DEFSYSV, '!bolo_flag_init', exist=flag_init
    if (flag_init NE 1) then begin
        print, mess.warning+"Since BOLO_INIT was not run before, we do it"
        BOLO_INIT
    endif
    ;; A partir d'ici, on est certain d'avoir au moins 3 bolo ...
    liste_bolo=['liste des bolo dispo',!bolo_list_name,'arret']
    nb_liste_bolo=N_ELEMENTS(liste_bolo)
    choix=WMENU(liste_bolo, init=1)
    if (choix EQ (nb_liste_bolo-1)) then begin
        print, mess.info+'you choise to Abort this program'
        return
    endif else begin
        ;; a ce jour, c'est fixe (pas joli du tout ...)
        if (choix EQ 1) then bolo=!bolo100
        if (choix EQ 2) then bolo=!bolo545
        if (choix EQ 3) then bolo=!bolo857
    endelse
    print, mess.info+"Effective Selected bolo is: .... ", bolo.name
endif
; --------------------------------------
; -E2: les niveaux de fond (background)
;
DEFSYSV, '!back_flag_init', exist=flag_init
if (flag_init NE 1) then begin
    print, mess.warning+"Since PLANCK_INIT_BACKGROUND was not run before, we do it"
    PLANCK_INIT_BACKGROUND
endif
; --------------------------------------
; -E3: les valeurs de flux de Jupiter
;
DEFSYSV, '!jupiter_flag_flux', exist=flag_init
if (flag_init NE 1) then begin
    print, mess.warning+"Since JUPITER_T2FLUX was not run before, we do it"
    JUPITER_T2FLUX
endif
;
; desormais, les 3 etapes initiales sont passees
; --------------------------------------
; ici, on regarde quel bolo est selectionne et on rassemble les infos
; On en profite pour s'arreter s'il n'est pas connu ! 
; (ceci peut arriver si le bolo est passe en parametre)
;
bolo_OK=0
if (bolo.freq EQ 100.) then begin
    background=!back.f100
    flux0=!Jupiter_flux.f100    ; MPiat : flux Jupiter a 100 GHz 1.6 e-12
    bolo_OK=1
endif
if (bolo.freq EQ 545.) then begin
    background=!back.f545       ; MPiat 10e-12
    flux0=!Jupiter_flux.f545    ; MPiat 1e-9
    bolo_OK=1
endif
if (bolo.freq EQ 857.) then begin
    background=!back.f857       ; MPiat 40e-12
    flux0=!Jupiter_flux.f857    ; MPiat 1e-9
    bolo_OK=1
endif
if (bolo_OK EQ 0) then begin
    print, mess.fatal+'no bolo structure found'
    return
endif
;
if (N_ELEMENTS(i_bolo) EQ 1) then begin
    print, mess.warning+"default I_polar Bolo value was (locally) changed"
    bolo.i_polar=i_bolo
endif
;
;flux0=0. ;pour test
;
flux0=flux0*1000.
;
flux1=flux0/100.
flux2=flux0/10.
flux3=flux0/4.
;
;flux2=flux0*30.
;flux3=flux0*14.
;
; ---------------------------------------------
; largeur du lobe et vitesse de defilement ...
;
speed=6.*60.      ;; 6 degres par seconde (en arcmin/s)
width=5.         ;; 5 arc_min
;
; ---------------------------------------------
; nombre de points et pas d'echantillonnage
;
if (N_ELEMENTS(nbp) NE 1) then begin
    nbp=1e4          ;; soit 1 seconde complete
    delta_t=1e-4  ;; en seconde
endif else begin
    delta_t=1./FLOAT(nbp)
endelse
nbp=LONG(nbp)
;
if (N_ELEMENTS(background) NE 1) then background=1e-12
;
;if (N_ELEMENTS(flux) NE 1) then flux=1.e-9        ;; le flux total
at_time=nbp/2.*delta_t  ;; le centre de la source
;
; ---------------------------------------------
; A partir d'ici, toutes les initilisations ont ete faites
; ---------------------------------------------
; si demander on affiche les principales info
;
if KEYWORD_SET(bavard) then begin
    print, mess.info+' Bolo Name : ..............', bolo.freq
    print, mess.info+' Backgroud : ..............', background
    print, mess.info+' Jupiter Flux : ...........', flux0
    print, mess.info+' Polarisation current : ... NOT READY' ;, i_bolo
endif
;
; ---------------------------------------------
; debut des calculs
;
noise=1e-16
if (N_ELEMENTS(noise) EQ 1) then begin
    noise=RANDOMN(seed,nbp)*noise
endif else begin
    noise=0.
endelse
;
input0=BEAM_GAUSS(nbp, delta_t=delta_t, flux0, width, at_time, $
                  speed=speed)
input0=input0+noise
EQUA_TEMP_BOLO, delta_t=delta_t, bolo=bolo, $
  signal=input0, $ ;; signal d'entree (le beam)
  background=background, $  ;; niveau du fond (number en W)
  TempOut=TempOut0              ;,/debu ;; signal de sortie (beam + reponse bolo)
;
input1=BEAM_GAUSS(nbp, delta_t=delta_t, flux1, width, at_time, $
                  speed=speed)
input1=input1+noise
EQUA_TEMP_BOLO, delta_t=delta_t, bolo=bolo, $
  signal=input1, $ ;; signal d'entree (le beam)
  background=background, $  ;; niveau du fond (number en W)
  TempOut=TempOut1 ;; signal de sortie (beam + reponse bolo)
;
input2=BEAM_GAUSS(nbp, delta_t=delta_t, flux2, width, at_time, $
                  speed=speed)
input2=input2+noise
EQUA_TEMP_BOLO, delta_t=delta_t, bolo=bolo, $
  signal=input2, $ ;; signal d'entree (le beam)
  background=background, $  ;; niveau du fond (number en W)
  TempOut=TempOut2 ;; signal de sortie (beam + reponse bolo)
;
input3=BEAM_GAUSS(nbp, delta_t=delta_t, flux3, width, at_time, $
                  speed=speed)
input3=input3+noise
EQUA_TEMP_BOLO, delta_t=delta_t, bolo=bolo, $,
  signal=input3, $ ;; signal d'entree (le beam)
  background=background, $  ;; niveau du fond (number en W)
  TempOut=TempOut3 ;; signal de sortie (beam + reponse bolo)
;
; echelle de temps afin qu'on ramene tout entre [0, 1]
;
tt=delta_t*findgen(nbp)
;
; -------------------------------------------------------
; Affichage brut
;
if KEYWORD_SET(postscript) then begin
    iname=STRCOMPRESS('_i'+STRING(ROUND(bolo.i_polar/1.e-10)),/remove_all)
    filename=bolo.name+iname+'.ps'
    SET_PLOT, 'PS'
    orientation=0
    if (orientation EQ 1) then begin
        device,/color, bits_per_pix=8, filename=filename,/portrait, $
          xsize=20, ysize=28, xoffset=0.5, yoffset=0.5
        !p.multi=[2,0,2]
    endif else begin
        device,/color, bits_per_pix=8, filename=filename,/portrait, $
          xsize=28, ysize=15, xoffset=0.5, yoffset=0.5
        !p.multi=[2,2,0]
        !p.charsize=0.8
    endelse
endif else begin
    Window, 2
endelse
;
if (N_ELEMENTS(xrange) NE 2) then xrange=[.475,.55]
;
plot, tt,  tempout0, xrange=xrange, /xstyle, /nodata, /ynozero, $
  xtitle='Time (s)', ytitle='Temperature (K)', $
  title='Temperature versus Time'
oplot, tt, tempout0, color=!col_o
oplot, tt, tempout2, color=!col_r
oplot, tt, tempout3, color=!col_g
oplot, tt, tempout1
;
plots, [0.5,0.5], !y.crange, linestyle=2
;
; -------------------------------------------------------
; Affichage (renormalise)
;
if NOT(KEYWORD_SET(postscript)) then Window,0
;
plot,tt,  tempout1-tempout1[nbp-1], xrange=xrange, /xstyle, $ ; yr=[-1,1]*1e-5,$
  xtitle='Time (s)', ytitle='Renormalized Temperature (K)', $
  title='Re-Normalized Temperature versus Time to Jupiter/100 curve'
oplot,tt,(tempout2-tempout2[nbp-1])/(flux2/flux1),color=!col_r
oplot,tt,(tempout3-tempout3[nbp-1])/(flux3/flux1),color=!col_g
oplot,tt,(tempout0-tempout0[nbp-1])/(flux0/flux1),color=!col_o
;
plots, [0.5,0.5], !y.crange, linestyle=2
;
; on trace des traits de couleur et on ecrit des commentaires !
;
xmin=!x.crange[0]
xmax=!x.crange[1]
xdyna=xmax-xmin
ymin=!y.crange[0]
ymax=!y.crange[1]
ydyna=ymax-ymin
xdebut=xmin+0.6*xdyna
xfin=xmin+0.7*xdyna
ylevel=ymin+[0.7,0.75,0.8,0.85]*ydyna
xoffset=0.02*xdyna
;
PLOTS, [xdebut, xfin], [ylevel(0),ylevel(0)], color=!col_r
XYOUTS, xfin+xoffset, ylevel(0), 'Flux 2 (Jup/10)'
;
PLOTS, [xdebut, xfin], [ylevel(1),ylevel(1)], color=!col_g
XYOUTS, xfin+xoffset, ylevel(1), 'Flux 3 (Jup/4)'
;
PLOTS, [xdebut, xfin], [ylevel(2),ylevel(2)], color=!col_o
XYOUTS, xfin+xoffset, ylevel(2), 'Flux 0 (Jupiter)'
;
PLOTS, [xdebut, xfin], [ylevel(3),ylevel(3)]
XYOUTS, xfin+xoffset, ylevel(3), 'Flux Ref (Jup/100)'
;
; autres informations importantes
;
xpos=xdebut  ;; was xfin+xoffset
XYOUTS, xpos, ylevel(0)-0.05*ydyna, 'Bolo name : '+bolo.name
XYOUTS, xpos, ylevel(0)-0.10*ydyna, 'Current :'+STRING(bolo.i_polar)
XYOUTS, xpos, ylevel(0)-0.15*ydyna, 'Background :'+STRING(background)
XYOUTS, xpos, ylevel(0)-0.20*ydyna, 'Jupiter flux :'+STRING(flux0)
XYOUTS, xpos, ylevel(0)-0.25*ydyna, 'npb integration :'+STRING(nbp)
;
if KEYWORD_SET(postscript) then begin
    device,/close
    !p.multi=0
    SET_PLOT, 'X'
endif 
;
if KEYWORD_SET(test) then STOP
;
end


