;
; Creation par AC le 17/01/2004 a Meudon
;
; Description
;  Soit une source dans le ciel de largeur angulaire reelle "width"
;  Soit une vitesse de defilement du satellite "speed"
;  Ce programme calcule un lobe gaussien de largeur liee
;  a la vitesse de defilement et a la largeur angulaire reelle des sources.
; 
; Historique des modifications:
;
; - 06/05/2005: - ajout et gestion de /Help
;               - renommee de BEAM_GAUSS en HFI_BEAM_GAUSS
; - fin Janvier 2004 : mise a plat avec JML de la formule donnant la largeur
;
; Dependances (Ce programme necessite):
;      AC_SETMESSAGES 
;
function HFI_BEAM_GAUSS, nbp, flux, width, at_time, $
                         delta_t=delta_t, speed=speed, $
                         test=test, help=help
;
mess=AC_SETMESSAGES("HFI_BEAM_GAUSS")
;
if (N_PARAMS() lt 4) then begin
	print, mess.fatal+'missing arguments'
	help=1
endif
;
if KEYWORD_SET(help) then begin
    print, 'function HFI_BEAM_GAUSS, nbp, flux, width, at_time, $'
    print, '                         delta_t=delta_t, speed=speed, $'
    print, '                         test=test, help=help'
    print, ''
    print, 'This function computes a time line containing a Gaussian beam'
    print, 'following Planck scanning speed'
    print, ''
    print, 'CALLING SEQUENCE : data=HFI_BEAM_GAUSS(nbp, flux, width, at_time)'
    return, -1
endif
;
if (N_ELEMENTS(delta_t) NE 1) then delta_t=1e-4
if (delta_t LE 0.) then begin
	print, mess.fatal+'"delta_t" must be positive'
	return, -1
endif
;
;instants=FINDGEN(nbp)*delta_t
instants=DINDGEN(nbp)*delta_t
;
largeur=width/speed
;
sqt2=(largeur^2)/(4.*ALOG(2.))
valeurs=-(instants-at_time)^2/sqt2
;
; on met tout a Zero dans le tableau de sortie
;
beam_profil=DBLARR(nbp)
;
; on ne calcule l'exponentiel que lorsque les valeurs
; sont au-dessus d'un seuil, ici ExpSeuil=-20
; 1/ on recherche la ou on est > ExpSeuil
ExpSeuil=-20.D
indices_OK=WHERE(valeurs GT ExpSeuil, nbp_OK)
; 2/ on ne calcule que la ou ca en vaut la peine
if (nbp_OK GT 0) then beam_profil(indices_OK)=flux*EXP(valeurs(indices_OK))
;
if KEYWORD_SET(test) then STOP
;
return, beam_profil
;
end
